@@ -37,7 +37,7 @@ import Text.Pandoc.Shared ( tabFilter, readDataFileUTF8, safeRead,
                             headerShift, normalize, err, warn )
 import Text.Pandoc.XML ( toEntities, fromEntities )
 import Text.Pandoc.SelfContained ( makeSelfContained )
-import Text.Pandoc.Highlighting ( languages, Style, tango, pygments,
+import Text.Highlighting.Kate ( languages, Style, tango, pygments,
          espresso, zenburn, kate, haddock, monochrome )
 import System.Environment ( getArgs, getProgName )
 import System.Exit ( exitWith, ExitCode (..) )
@@ -69,7 +69,8 @@ compileInfo =
   "\nCompiled with citeproc-hs " ++ VERSION_citeproc_hs ++ ", texmath " ++
   VERSION_texmath ++ ", highlighting-kate " ++ VERSION_highlighting_kate ++
    ".\nSyntax highlighting is supported for the following languages:\n    " ++
-       wrapWords 4 78 languages
+       wrapWords 4 78
+       [map toLower l | l <- languages, l /= "Alert" && l /= "Alert_indent"]
 
 -- | Converts a list of strings into a single string with the items printed as
 -- comma separated words in lines with a maximum line length.
@@ -99,6 +100,7 @@ data Opt = Opt
     , optVariables         :: [(String,String)] -- ^ Template variables to set
     , optOutputFile        :: String  -- ^ Name of output file
     , optNumberSections    :: Bool    -- ^ Number sections in LaTeX
+    , optNumberOffset      :: [Int]   -- ^ Starting number for sections
     , optSectionDivs       :: Bool    -- ^ Put sections in div tags in HTML
     , optIncremental       :: Bool    -- ^ Use incremental lists in Slidy/Slideous/S5
     , optSelfContained     :: Bool    -- ^ Make HTML accessible offline
@@ -137,61 +139,64 @@ data Opt = Opt
     , optSetextHeaders     :: Bool       -- ^ Use atx headers for markdown level 1-2
     , optAscii             :: Bool       -- ^ Use ascii characters only in html
     , optTeXLigatures      :: Bool       -- ^ Use TeX ligatures for quotes/dashes
+    , optDefaultImageExtension :: String -- ^ Default image extension
     }
 
 -- | Defaults for command-line options.
 defaultOpts :: Opt
 defaultOpts = Opt
-    { optTabStop           = 4
-    , optPreserveTabs      = False
-    , optStandalone        = False
-    , optReader            = ""    -- null for default reader
-    , optWriter            = ""    -- null for default writer
-    , optParseRaw          = False
-    , optTableOfContents   = False
-    , optTransforms        = []
-    , optTemplate          = Nothing
-    , optVariables         = []
-    , optOutputFile        = "-"    -- "-" means stdout
-    , optNumberSections    = False
-    , optSectionDivs       = False
-    , optIncremental       = False
-    , optSelfContained     = False
-    , optSmart             = False
-    , optOldDashes         = False
-    , optHtml5             = False
-    , optHtmlQTags         = False
-    , optHighlight         = True
-    , optHighlightStyle    = pygments
-    , optChapters          = False
-    , optHTMLMathMethod    = PlainMath
-    , optReferenceODT      = Nothing
-    , optReferenceDocx     = Nothing
-    , optEpubStylesheet    = Nothing
-    , optEpubMetadata      = ""
-    , optEpubFonts         = []
-    , optEpubChapterLevel  = 1
-    , optTOCDepth          = 3
-    , optDumpArgs          = False
-    , optIgnoreArgs        = False
-    , optReferenceLinks    = False
-    , optWrapText          = True
-    , optColumns           = 72
-    , optPlugins           = []
-    , optEmailObfuscation  = JavascriptObfuscation
-    , optIdentifierPrefix  = ""
-    , optIndentedCodeClasses = []
-    , optDataDir           = Nothing
-    , optCiteMethod        = Citeproc
-    , optBibliography      = []
-    , optCslFile           = Nothing
-    , optAbbrevsFile       = Nothing
-    , optListings          = False
-    , optLaTeXEngine       = "pdflatex"
-    , optSlideLevel        = Nothing
-    , optSetextHeaders     = True
-    , optAscii             = False
-    , optTeXLigatures      = True
+    { optTabStop               = 4
+    , optPreserveTabs          = False
+    , optStandalone            = False
+    , optReader                = ""    -- null for default reader
+    , optWriter                = ""    -- null for default writer
+    , optParseRaw              = False
+    , optTableOfContents       = False
+    , optTransforms            = []
+    , optTemplate              = Nothing
+    , optVariables             = []
+    , optOutputFile            = "-"    -- "-" means stdout
+    , optNumberSections        = False
+    , optNumberOffset          = [0,0,0,0,0,0]
+    , optSectionDivs           = False
+    , optIncremental           = False
+    , optSelfContained         = False
+    , optSmart                 = False
+    , optOldDashes             = False
+    , optHtml5                 = False
+    , optHtmlQTags             = False
+    , optHighlight             = True
+    , optHighlightStyle        = pygments
+    , optChapters              = False
+    , optHTMLMathMethod        = PlainMath
+    , optReferenceODT          = Nothing
+    , optReferenceDocx         = Nothing
+    , optEpubStylesheet        = Nothing
+    , optEpubMetadata          = ""
+    , optEpubFonts             = []
+    , optEpubChapterLevel      = 1
+    , optTOCDepth              = 3
+    , optDumpArgs              = False
+    , optIgnoreArgs            = False
+    , optReferenceLinks        = False
+    , optWrapText              = True
+    , optColumns               = 72
+    , optPlugins               = []
+    , optEmailObfuscation      = JavascriptObfuscation
+    , optIdentifierPrefix      = ""
+    , optIndentedCodeClasses   = []
+    , optDataDir               = Nothing
+    , optCiteMethod            = Citeproc
+    , optBibliography          = []
+    , optCslFile               = Nothing
+    , optAbbrevsFile           = Nothing
+    , optListings              = False
+    , optLaTeXEngine           = "pdflatex"
+    , optSlideLevel            = Nothing
+    , optSetextHeaders         = True
+    , optAscii                 = False
+    , optTeXLigatures          = True
+    , optDefaultImageExtension = ""
     }
 
 -- | A list of functions, each transforming the options data structure
@@ -464,6 +469,16 @@ options =
                   (\opt -> return opt { optNumberSections = True }))
                  "" -- "Number sections in LaTeX"
 
+    , Option "" ["number-offset"]
+                 (ReqArg
+                  (\arg opt ->
+                      case safeRead ('[':arg ++ "]") of
+                           Just ns -> return opt { optNumberOffset = ns,
+                                                   optNumberSections = True }
+                           _      -> err 57 "could not parse number-offset")
+                 "NUMBERS")
+                 "" -- "Starting number for sections, subsections, etc."
+
     , Option "" ["no-tex-ligatures"]
                  (NoArg
                   (\opt -> return opt { optTeXLigatures = False }))
@@ -495,6 +510,12 @@ options =
                   (\opt -> return opt { optSectionDivs = True }))
                  "" -- "Put sections in div tags in HTML"
 
+    , Option "" ["default-image-extension"]
+                 (ReqArg
+                  (\arg opt -> return opt { optDefaultImageExtension = arg })
+                   "extension")
+                  "" -- "Default extension for extensionless images"
+
     , Option "" ["email-obfuscation"]
                  (ReqArg
                   (\arg opt -> do
@@ -696,8 +717,10 @@ options =
                  (NoArg
                   (\_ -> do
                      prg <- getProgName
-                     UTF8.hPutStrLn stdout (prg ++ " " ++ pandocVersion ++ compileInfo ++
-                                       copyrightMessage)
+                     defaultDatadir <- getAppUserDataDirectory "pandoc"
+                     UTF8.hPutStrLn stdout (prg ++ " " ++ pandocVersion ++
+                       compileInfo ++ "\nDefault user data directory: " ++
+                       defaultDatadir ++ copyrightMessage)
                      exitWith ExitSuccess ))
                  "" -- "Print version"
 
@@ -806,55 +829,57 @@ main = do
   -- thread option data structure through all supplied option actions
   opts <- foldl (>>=) (return defaultOpts') actions
 
-  let Opt    {  optTabStop           = tabStop
-              , optPreserveTabs      = preserveTabs
-              , optStandalone        = standalone
-              , optReader            = readerName
-              , optWriter            = writerName
-              , optParseRaw          = parseRaw
-              , optVariables         = variables
-              , optTableOfContents   = toc
-              , optTransforms        = transforms
-              , optTemplate          = templatePath
-              , optOutputFile        = outputFile
-              , optNumberSections    = numberSections
-              , optSectionDivs       = sectionDivs
-              , optIncremental       = incremental
-              , optSelfContained     = selfContained
-              , optSmart             = smart
-              , optOldDashes         = oldDashes
-              , optHtml5             = html5
-              , optHtmlQTags         = htmlQTags
-              , optHighlight         = highlight
-              , optHighlightStyle    = highlightStyle
-              , optChapters          = chapters
-              , optHTMLMathMethod    = mathMethod
-              , optReferenceODT      = referenceODT
-              , optReferenceDocx     = referenceDocx
-              , optEpubStylesheet    = epubStylesheet
-              , optEpubMetadata      = epubMetadata
-              , optEpubFonts         = epubFonts
-              , optEpubChapterLevel  = epubChapterLevel
-              , optTOCDepth          = epubTOCDepth
-              , optDumpArgs          = dumpArgs
-              , optIgnoreArgs        = ignoreArgs
-              , optReferenceLinks    = referenceLinks
-              , optWrapText          = wrap
-              , optColumns           = columns
-              , optEmailObfuscation  = obfuscationMethod
-              , optIdentifierPrefix  = idPrefix
-              , optIndentedCodeClasses = codeBlockClasses
-              , optDataDir           = mbDataDir
-              , optBibliography      = reffiles
-              , optCslFile           = mbCsl
-              , optAbbrevsFile       = cslabbrevs
-              , optCiteMethod        = citeMethod
-              , optListings          = listings
-              , optLaTeXEngine       = latexEngine
-              , optSlideLevel        = slideLevel
-              , optSetextHeaders     = setextHeaders
-              , optAscii             = ascii
-              , optTeXLigatures      = texLigatures
+  let Opt    {  optTabStop               = tabStop
+              , optPreserveTabs          = preserveTabs
+              , optStandalone            = standalone
+              , optReader                = readerName
+              , optWriter                = writerName
+              , optParseRaw              = parseRaw
+              , optVariables             = variables
+              , optTableOfContents       = toc
+              , optTransforms            = transforms
+              , optTemplate              = templatePath
+              , optOutputFile            = outputFile
+              , optNumberSections        = numberSections
+              , optNumberOffset            = numberFrom
+              , optSectionDivs           = sectionDivs
+              , optIncremental           = incremental
+              , optSelfContained         = selfContained
+              , optSmart                 = smart
+              , optOldDashes             = oldDashes
+              , optHtml5                 = html5
+              , optHtmlQTags             = htmlQTags
+              , optHighlight             = highlight
+              , optHighlightStyle        = highlightStyle
+              , optChapters              = chapters
+              , optHTMLMathMethod        = mathMethod
+              , optReferenceODT          = referenceODT
+              , optReferenceDocx         = referenceDocx
+              , optEpubStylesheet        = epubStylesheet
+              , optEpubMetadata          = epubMetadata
+              , optEpubFonts             = epubFonts
+              , optEpubChapterLevel      = epubChapterLevel
+              , optTOCDepth              = epubTOCDepth
+              , optDumpArgs              = dumpArgs
+              , optIgnoreArgs            = ignoreArgs
+              , optReferenceLinks        = referenceLinks
+              , optWrapText              = wrap
+              , optColumns               = columns
+              , optEmailObfuscation      = obfuscationMethod
+              , optIdentifierPrefix      = idPrefix
+              , optIndentedCodeClasses   = codeBlockClasses
+              , optDataDir               = mbDataDir
+              , optBibliography          = reffiles
+              , optCslFile               = mbCsl
+              , optAbbrevsFile           = cslabbrevs
+              , optCiteMethod            = citeMethod
+              , optListings              = listings
+              , optLaTeXEngine           = latexEngine
+              , optSlideLevel            = slideLevel
+              , optSetextHeaders         = setextHeaders
+              , optAscii                 = ascii
+              , optTeXLigatures          = texLigatures
+              , optDefaultImageExtension = defaultImageExtension
              } = opts
 
   when dumpArgs $
@@ -996,6 +1021,7 @@ main = do
                       , readerCitationStyle = mbsty
                       , readerIndentedCodeClasses = codeBlockClasses
                       , readerApplyMacros = not laTeXOutput
+                      , readerDefaultImageExtension = defaultImageExtension
                       }
 
   let writerOptions = def { writerStandalone       = standalone',
@@ -1009,6 +1035,7 @@ main = do
                             writerBiblioFiles      = reffiles,
                             writerIgnoreNotes      = False,
                             writerNumberSections   = numberSections,
+                            writerNumberOffset     = numberFrom,
                             writerSectionDivs      = sectionDivs,
                             writerReferenceLinks   = referenceLinks,
                             writerWrapText         = wrap,
