@@ -204,22 +204,24 @@ data ReaderOptions = ReaderOptions{
        , readerApplyMacros     :: Bool -- ^ Apply macros to TeX math
        , readerIndentedCodeClasses :: [String] -- ^ Default classes for
                                        -- indented code blocks
+       , readerDefaultImageExtension :: String -- ^ Default extension for images
 } deriving (Show, Read)
 
 instance Default ReaderOptions
   where def = ReaderOptions{
-                 readerExtensions          = pandocExtensions
-               , readerSmart               = False
-               , readerStrict              = False
-               , readerStandalone          = False
-               , readerParseRaw            = False
-               , readerColumns             = 80
-               , readerTabStop             = 4
-               , readerOldDashes           = False
-               , readerReferences          = []
-               , readerCitationStyle       = Nothing
-               , readerApplyMacros         = True
-               , readerIndentedCodeClasses = []
+                 readerExtensions            = pandocExtensions
+               , readerSmart                 = False
+               , readerStrict                = False
+               , readerStandalone            = False
+               , readerParseRaw              = False
+               , readerColumns               = 80
+               , readerTabStop               = 4
+               , readerOldDashes             = False
+               , readerReferences            = []
+               , readerCitationStyle         = Nothing
+               , readerApplyMacros           = True
+               , readerIndentedCodeClasses   = []
+               , readerDefaultImageExtension = ""
                }
 
 --
@@ -268,6 +270,7 @@ data WriterOptions = WriterOptions
   , writerHTMLMathMethod   :: HTMLMathMethod  -- ^ How to print math in HTML
   , writerIgnoreNotes      :: Bool   -- ^ Ignore footnotes (used in making toc)
   , writerNumberSections   :: Bool   -- ^ Number sections in LaTeX
+  , writerNumberOffset     :: [Int]  -- ^ Starting number for section, subsection, ...
   , writerSectionDivs      :: Bool   -- ^ Put sections in div tags in HTML
   , writerExtensions       :: Set Extension -- ^ Markdown extensions that can be used
   , writerReferenceLinks   :: Bool   -- ^ Use reference links in writing markdown, rst
@@ -311,6 +314,7 @@ instance Default WriterOptions where
                       , writerHTMLMathMethod   = PlainMath
                       , writerIgnoreNotes      = False
                       , writerNumberSections   = False
+                      , writerNumberOffset     = [0,0,0,0,0,0]
                       , writerSectionDivs      = False
                       , writerExtensions       = pandocExtensions
                       , writerReferenceLinks   = False
