@@ -149,11 +149,12 @@ where
 import Text.Pandoc.Definition
 import Text.Pandoc.Options
 import Text.Pandoc.Builder (Blocks, Inlines, rawBlock)
+import Text.Pandoc.XML (fromEntities)
 import qualified Text.Pandoc.UTF8 as UTF8 (putStrLn)
 import Text.Parsec
 import Text.Parsec.Pos (newPos)
-import Data.Char ( toLower, toUpper, ord, isAscii, isAlphaNum, isDigit, isHexDigit,
-                   isSpace )
+import Data.Char ( toLower, toUpper, ord, chr, isAscii, isAlphaNum, isDigit,
+                   isHexDigit, isSpace )
 import Data.List ( intercalate, transpose )
 import Text.Pandoc.Shared
 import qualified Data.Map as M
@@ -244,7 +245,13 @@ oneOfStrings = oneOfStrings' (==)
 -- | Parses one of a list of strings (tried in order), case insensitive.
 oneOfStringsCI :: [String] -> Parser [Char] st String
 oneOfStringsCI = oneOfStrings' ciMatch
-  where ciMatch x y = toLower x == toLower y
+  where ciMatch x y = toLower' x == toLower' y
+        -- this optimizes toLower by checking common ASCII case
+        -- first, before calling the expensive unicode-aware
+        -- function:
+        toLower' c | c >= 'A' && c <= 'Z' = chr (ord c + 32)
+                   | isAscii c = c
+                   | otherwise = toLower c
 
 -- | Parses a space or tab.
 spaceChar :: Parser [Char] st Char
@@ -363,7 +370,7 @@ romanNumeral upperCase = do
 -- escaped mailto: URI.
 emailAddress :: Parser [Char] st (String, String)
 emailAddress = try $ liftA2 toResult mailbox (char '@' *> domain)
- where toResult mbox dom = let full = mbox ++ '@':dom
+ where toResult mbox dom = let full = fromEntities $ mbox ++ '@':dom
                            in  (full, escapeURI $ "mailto:" ++ full)
        mailbox           = intercalate "." `fmap` (emailWord `sepby1` dot)
        domain            = intercalate "." `fmap` (subdomain `sepby1` dot)
@@ -373,7 +380,7 @@ emailAddress = try $ liftA2 toResult mailbox (char '@' *> domain)
                                  notFollowedBy space)
        emailWord         = many1 $ satisfy isEmailChar
        isEmailChar c     = isAlphaNum c || isEmailPunct c
-       isEmailPunct c    = c `elem` "!\"#$%&'*+-/=?^_{|}~"
+       isEmailPunct c    = c `elem` "!\"#$%&'*+-/=?^_{|}~;"
        -- note: sepBy1 from parsec consumes input when sep
        -- succeeds and p fails, so we use this variant here.
        sepby1 p sep      = liftA2 (:) p (many (try $ sep >> p))
@@ -434,7 +441,7 @@ uri = try $ do
                                          <|> enclosed (char '[') (char ']') uriChunk)
                                          <|> uriChunk))
   str' <- option str $ char '/' >> return (str ++ "/")
-  let uri' = scheme ++ ":" ++ str'
+  let uri' = scheme ++ ":" ++ fromEntities str'
   return (uri', escapeURI uri')
 
 -- | Applies a parser, returns tuple of its results and its horizontal
@@ -787,7 +794,7 @@ data ParserState = ParserState
       stateAuthors         :: [[Inline]],    -- ^ Authors of document
       stateDate            :: [Inline],      -- ^ Date of document
       stateHeaderTable     :: [HeaderType],  -- ^ Ordered list of header types used
-      stateHeaders         :: [[Inline]],    -- ^ List of headers (used for implicit ref links)
+      stateHeaders         :: M.Map Inlines String, -- ^ List of headers and ids (used for implicit ref links)
       stateIdentifiers     :: [String],      -- ^ List of header identifiers used
       stateNextExample     :: Int,           -- ^ Number of next example
       stateExamples        :: M.Map String Int, -- ^ Map from example labels to numbers
@@ -816,7 +823,7 @@ defaultParserState =
                   stateAuthors         = [],
                   stateDate            = [],
                   stateHeaderTable     = [],
-                  stateHeaders         = [],
+                  stateHeaders         = M.empty,
                   stateIdentifiers     = [],
                   stateNextExample     = 1,
                   stateExamples        = M.empty,
