@@ -51,6 +51,8 @@ import System.FilePath (replaceExtension, (</>))
 import Data.List (intercalate)
 import qualified Data.Map as M
 import qualified Control.Exception as E
+import System.FilePath (takeExtension, addExtension)
+import Text.Pandoc.Highlighting (fromListingsLanguage)
 
 -- | Parse LaTeX from string and return 'Pandoc' document.
 readLaTeX :: ReaderOptions -- ^ Reader options
@@ -255,19 +257,28 @@ blockCommands = M.fromList $
   , ("signature", mempty <$ (skipopts *> authors))
   , ("date", mempty <$ (skipopts *> tok >>= addDate))
   -- sectioning
-  , ("chapter", updateState (\s -> s{ stateHasChapters = True }) *> section 0)
-  , ("section", section 1)
-  , ("subsection", section 2)
-  , ("subsubsection", section 3)
-  , ("paragraph", section 4)
-  , ("subparagraph", section 5)
+  , ("chapter", updateState (\s -> s{ stateHasChapters = True })
+                      *> section nullAttr 0)
+  , ("chapter*", updateState (\s -> s{ stateHasChapters = True })
+                      *> section ("",["unnumbered"],[]) 0)
+  , ("section", section nullAttr 1)
+  , ("section*", section ("",["unnumbered"],[]) 1)
+  , ("subsection", section nullAttr 2)
+  , ("subsection*", section ("",["unnumbered"],[]) 2)
+  , ("subsubsection", section nullAttr 3)
+  , ("subsubsection*", section ("",["unnumbered"],[]) 3)
+  , ("paragraph", section nullAttr 4)
+  , ("paragraph*", section ("",["unnumbered"],[]) 4)
+  , ("subparagraph", section nullAttr 5)
+  , ("subparagraph*", section ("",["unnumbered"],[]) 5)
   -- beamer slides
-  , ("frametitle", section 3)
-  , ("framesubtitle", section 4)
+  , ("frametitle", section nullAttr 3)
+  , ("framesubtitle", section nullAttr 4)
   -- letters
   , ("opening", (para . trimInlines) <$> (skipopts *> tok))
   , ("closing", skipopts *> closing)
   --
+  , ("hrule", pure horizontalRule)
   , ("rule", skipopts *> tok *> tok *> pure horizontalRule)
   , ("item", skipopts *> loose_item)
   , ("documentclass", skipopts *> braced *> preamble)
@@ -311,13 +322,13 @@ authors = try $ do
 addDate :: Inlines -> LP ()
 addDate dat = updateState (\s -> s{ stateDate = toList dat })
 
-section :: Int -> LP Blocks
-section lvl = do
+section :: Attr -> Int -> LP Blocks
+section attr lvl = do
   hasChapters <- stateHasChapters `fmap` getState
   let lvl' = if hasChapters then lvl + 1 else lvl
   skipopts
   contents <- grouped inline
-  return $ header lvl' contents
+  return $ headerWith attr lvl' contents
 
 inlineCommand :: LP Inlines
 inlineCommand = try $ do
@@ -430,8 +441,7 @@ inlineCommands = M.fromList $
   , ("href", (unescapeURL <$> braced <* optional sp) >>= \url ->
        tok >>= \lab ->
          pure (link url "" lab))
-  , ("includegraphics", skipopts *> (unescapeURL <$> braced) >>=
-       (\src -> pure (image src "" (str "image"))))
+  , ("includegraphics", skipopts *> (unescapeURL <$> braced) >>= mkImage)
   , ("enquote", enquote)
   , ("cite", citation "cite" AuthorInText False)
   , ("citep", citation "citep" NormalCitation False)
@@ -488,6 +498,21 @@ inlineCommands = M.fromList $
   -- in which case they will appear as raw latex blocks:
   [ "noindent", "index", "nocite" ]
 
+mkImage :: String -> LP Inlines
+mkImage src = do
+   -- try for a caption
+   (alt, tit) <- option (str "image", "") $ try $ do
+                   spaces
+                   controlSeq "caption"
+                   optional (char '*')
+                   ils <- grouped inline
+                   return (ils, "fig:")
+   case takeExtension src of
+        "" -> do
+              defaultExt <- getOption readerDefaultImageExtension
+              return $ image (addExtension src defaultExt) tit alt
+        _  -> return $ image src tit alt
+
 inNote :: Inlines -> Inlines
 inNote ils =
   note $ para $ ils <> str "."
@@ -744,6 +769,19 @@ verbCmd = do
   rest <- getInput
   return (r, rest)
 
+keyval :: LP (String, String)
+keyval = try $ do
+  key <- many1 alphaNum
+  val <- option "" $ char '=' >> many1 alphaNum
+  skipMany spaceChar
+  optional (char ',')
+  skipMany spaceChar
+  return (key, val)
+
+
+keyvals :: LP [(String, String)]
+keyvals = try $ char '[' *> manyTill keyval (char ']')
+
 verbatimEnv :: LP (String, String)
 verbatimEnv = do
   (_,r) <- withRaw $ do
@@ -779,10 +817,34 @@ environments = M.fromList
       (codeBlockWith ("",["sourceCode","literate","haskell"],[]) <$>
         verbEnv "code"))
   , ("verbatim", codeBlock <$> (verbEnv "verbatim"))
-  , ("Verbatim", codeBlock <$> (verbEnv "Verbatim"))
-  , ("lstlisting", codeBlock <$> (verbEnv "lstlisting"))
-  , ("minted", liftA2 (\l c -> codeBlockWith ("",[l],[]) c)
-            (grouped (many1 $ satisfy (/= '}'))) (verbEnv "minted"))
+  , ("Verbatim",   do options <- option [] keyvals
+                      let kvs = [ (if k == "firstnumber"
+                                      then "startFrom"
+                                      else k, v) | (k,v) <- options ]
+                      let classes = [ "numberLines" |
+                                      lookup "numbers" options == Just "left" ]
+                      let attr = ("",classes,kvs)
+                      codeBlockWith attr <$> (verbEnv "Verbatim"))
+  , ("lstlisting", do options <- option [] keyvals
+                      let kvs = [ (if k == "firstnumber"
+                                      then "startFrom"
+                                      else k, v) | (k,v) <- options ]
+                      let classes = [ "numberLines" |
+                                      lookup "numbers" options == Just "left" ]
+                                 ++ maybe [] (:[]) (lookup "language" options
+                                         >>= fromListingsLanguage)
+                      let attr = ("",classes,kvs)
+                      codeBlockWith attr <$> (verbEnv "lstlisting"))
+  , ("minted",     do options <- option [] keyvals
+                      lang <- grouped (many1 $ satisfy (/='}'))
+                      let kvs = [ (if k == "firstnumber"
+                                      then "startFrom"
+                                      else k, v) | (k,v) <- options ]
+                      let classes = [ lang | not (null lang) ] ++
+                                    [ "numberLines" |
+                                      lookup "linenos" options == Just "true" ]
+                      let attr = ("",classes,kvs)
+                      codeBlockWith attr <$> (verbEnv "minted"))
   , ("obeylines", parseFromString
                   (para . trimInlines . mconcat <$> many inline) =<<
                   intercalate "\\\\\n" . lines <$> verbEnv "obeylines")
@@ -888,10 +950,10 @@ ordered_list = do
 
 paragraph :: LP Blocks
 paragraph = do
-  x <- mconcat <$> many1 inline
+  x <- trimInlines . mconcat <$> many1 inline
   if x == mempty
      then return mempty
-     else return $ para $ trimInlines x
+     else return $ para x
 
 preamble :: LP Blocks
 preamble = mempty <$> manyTill preambleBlock beginDoc
