@@ -41,16 +41,17 @@ import qualified Text.Pandoc.Builder as B
 import Text.Pandoc.Builder (Inlines, Blocks, trimInlines, (<>))
 import Text.Pandoc.Options
 import Text.Pandoc.Shared
+import Text.Pandoc.XML (fromEntities)
 import Text.Pandoc.Parsing hiding (tableWith)
 import Text.Pandoc.Readers.LaTeX ( rawLaTeXInline, rawLaTeXBlock )
 import Text.Pandoc.Readers.HTML ( htmlTag, htmlInBalanced, isInlineTag, isBlockTag,
                                   isTextTag, isCommentTag )
-import Text.Pandoc.XML ( fromEntities )
 import Text.Pandoc.Biblio (processBiblio)
 import qualified Text.CSL as CSL
 import Data.Monoid (mconcat, mempty)
 import Control.Applicative ((<$>), (<*), (*>), (<$))
 import Control.Monad
+import System.FilePath (takeExtension, addExtension)
 import Text.HTML.TagSoup
 import Text.HTML.TagSoup.Match (tagOpen)
 import qualified Data.Set as Set
@@ -143,6 +144,7 @@ atMostSpaces n = (char ' ' >> atMostSpaces (n-1)) <|> return ()
 
 litChar :: MarkdownParser Char
 litChar = escapedChar'
+       <|> characterReference
        <|> noneOf "\n"
        <|> try (newline >> notFollowedBy blankline >> return ' ')
 
@@ -194,11 +196,14 @@ dateLine = try $ do
   trimInlinesF . mconcat <$> manyTill inline newline
 
 titleBlock :: MarkdownParser (F Inlines, F [Inlines], F Inlines)
-titleBlock = pandocTitleBlock <|> mmdTitleBlock
+titleBlock = pandocTitleBlock
+         <|> mmdTitleBlock
+         <|> return (mempty, return [], mempty)
 
 pandocTitleBlock :: MarkdownParser (F Inlines, F [Inlines], F Inlines)
 pandocTitleBlock = try $ do
   guardEnabled Ext_pandoc_title_block
+  lookAhead (char '%')
   title <- option mempty titleLine
   author <- option (return []) authorsLine
   date <- option mempty dateLine
@@ -260,7 +265,7 @@ referenceKey = try $ do
                     optional $ newline >> notFollowedBy blankline
                     skipMany spaceChar
                     notFollowedBy' (() <$ reference)
-                    many1 $ escapedChar' <|> satisfy (not . isBlank)
+                    many1 $ notFollowedBy space >> litChar
   let betweenAngles = try $ char '<' >>
                        manyTill (escapedChar' <|> litChar) (char '>')
   src <- try betweenAngles <|> sourceURL
@@ -282,8 +287,7 @@ referenceKey = try $ do
 referenceTitle :: MarkdownParser String
 referenceTitle = try $ do
   skipSpaces >> optional newline >> skipSpaces
-  let parenTit = charsInBalanced '(' ')' litChar
-  fromEntities <$> (quotedTitle '"' <|> quotedTitle '\'' <|> parenTit)
+  quotedTitle '"' <|> quotedTitle '\'' <|> charsInBalanced '(' ')' litChar
 
 -- A link title in quotes
 quotedTitle :: Char -> MarkdownParser String
@@ -291,7 +295,7 @@ quotedTitle c = try $ do
   char c
   notFollowedBy spaces
   let pEnder = try $ char c >> notFollowedBy (satisfy isAlphaNum)
-  let regChunk = many1 (noneOf ['\\','\n',c]) <|> count 1 litChar
+  let regChunk = many1 (noneOf ['\\','\n','&',c]) <|> count 1 litChar
   let nestedChunk = (\x -> [c] ++ x ++ [c]) <$> quotedTitle c
   unwords . words . concat <$> manyTill (nestedChunk <|> regChunk) pEnder
 
@@ -360,8 +364,8 @@ block = choice [ codeBlockFenced
                , lhsCodeBlock
                , rawTeXBlock
                , htmlBlock
-               , lineBlock
                , table
+               , lineBlock
                , codeBlockIndented
                , blockQuote
                , hrule
@@ -386,15 +390,22 @@ header = setextHeader <|> atxHeader <?> "header"
 --  returns unique identifier
 addToHeaderList :: Attr -> F Inlines -> MarkdownParser Attr
 addToHeaderList (ident,classes,kvs) text = do
-  let headerList = B.toList $ runF text defaultParserState
-  updateState $ \st -> st{ stateHeaders = headerList : stateHeaders st }
-  (do guardEnabled Ext_auto_identifiers
-      ids <- stateIdentifiers `fmap` getState
-      let id' = if null ident
-                   then uniqueIdent headerList ids
-                   else ident
-      updateState $ \st -> st{ stateIdentifiers = id' : ids }
-      return (id',classes,kvs)) <|> return ("",classes,kvs)
+  let header' = runF text defaultParserState
+  exts <- getOption readerExtensions
+  let insert' = M.insertWith (\_new old -> old)
+  if null ident && Ext_auto_identifiers `Set.member` exts
+     then do
+       ids <- stateIdentifiers `fmap` getState
+       let id' = uniqueIdent (B.toList header') ids
+       updateState $ \st -> st{
+              stateIdentifiers = id' : ids,
+              stateHeaders = insert' header' id' $ stateHeaders st }
+       return (id',classes,kvs)
+     else do
+        unless (null ident) $
+          updateState $ \st -> st{
+               stateHeaders = insert' header' ident $ stateHeaders st }
+        return (ident,classes,kvs)
 
 atxHeader :: MarkdownParser (F Blocks)
 atxHeader = try $ do
@@ -476,47 +487,48 @@ blockDelimiter f len = try $ do
       Nothing -> count 3 (char c) >> many (char c) >>=
                  return . (+ 3) . length
 
-attributes :: Parser [Char] st (String, [String], [(String, String)])
+attributes :: MarkdownParser Attr
 attributes = try $ do
   char '{'
   spnl
   attrs <- many (attribute >>~ spnl)
   char '}'
-  let (ids, classes, keyvals) = unzip3 attrs
-  let firstNonNull [] = ""
-      firstNonNull (x:xs) | not (null x) = x
-                          | otherwise    = firstNonNull xs
-  return (firstNonNull $ reverse ids, concat classes, concat keyvals)
+  return $ foldl (\x f -> f x) nullAttr attrs
 
-attribute :: Parser [Char] st (String, [String], [(String, String)])
-attribute = identifierAttr <|> classAttr <|> keyValAttr
+attribute :: MarkdownParser (Attr -> Attr)
+attribute = identifierAttr <|> classAttr <|> keyValAttr <|> specialAttr
 
-identifier :: Parser [Char] st String
+identifier :: MarkdownParser String
 identifier = do
   first <- letter
   rest <- many $ alphaNum <|> oneOf "-_:."
   return (first:rest)
 
-identifierAttr :: Parser [Char] st (String, [a], [a1])
+identifierAttr :: MarkdownParser (Attr -> Attr)
 identifierAttr = try $ do
   char '#'
   result <- identifier
-  return (result,[],[])
+  return $ \(_,cs,kvs) -> (result,cs,kvs)
 
-classAttr :: Parser [Char] st (String, [String], [a])
+classAttr :: MarkdownParser (Attr -> Attr)
 classAttr = try $ do
   char '.'
   result <- identifier
-  return ("",[result],[])
+  return $ \(id',cs,kvs) -> (id',cs ++ [result],kvs)
 
-keyValAttr :: Parser [Char] st (String, [a], [(String, String)])
+keyValAttr :: MarkdownParser (Attr -> Attr)
 keyValAttr = try $ do
   key <- identifier
   char '='
-  val <- enclosed (char '"') (char '"') anyChar
-     <|> enclosed (char '\'') (char '\'') anyChar
-     <|> many nonspaceChar
-  return ("",[],[(key,val)])
+  val <- enclosed (char '"') (char '"') litChar
+     <|> enclosed (char '\'') (char '\'') litChar
+     <|> many (escapedChar' <|> noneOf " \t\n\r}")
+  return $ \(id',cs,kvs) -> (id',cs,kvs ++ [(key,val)])
+
+specialAttr :: MarkdownParser (Attr -> Attr)
+specialAttr = do
+  char '-'
+  return $ \(id',cs,kvs) -> (id',cs ++ ["unnumbered"],kvs)
 
 codeBlockFenced :: MarkdownParser (F Blocks)
 codeBlockFenced = try $ do
@@ -1142,7 +1154,6 @@ pipeTable = try $ do
 
                           return (row, als) )
   lines' <- sequence <$> many1 pipeTableRow
-  blanklines
   let widths = replicate (length aligns) 0.0
   return $ (aligns, widths, heads, lines')
 
@@ -1173,10 +1184,12 @@ pipeTableRow = do
                       | otherwise   -> B.plain $ ils') cells'
 
 pipeTableHeaderPart :: Parser [Char] st Alignment
-pipeTableHeaderPart = do
+pipeTableHeaderPart = try $ do
+  skipMany spaceChar
   left <- optionMaybe (char ':')
   many1 (char '-')
   right <- optionMaybe (char ':')
+  skipMany spaceChar
   return $
     case (left,right) of
       (Nothing,Nothing) -> AlignDefault
@@ -1480,13 +1493,18 @@ reference :: MarkdownParser (F Inlines, String)
 reference = do notFollowedBy' (string "[^")   -- footnote reference
                withRaw $ trimInlinesF <$> inlinesInBalancedBrackets
 
+parenthesizedChars :: MarkdownParser [Char]
+parenthesizedChars = do
+  result <- charsInBalanced '(' ')' litChar
+  return $ '(' : result ++ ")"
+
 -- source for a link, with optional title
 source :: MarkdownParser (String, String)
 source = do
   char '('
   skipSpaces
   let urlChunk = try $ notFollowedBy (oneOf "\"')") >>
-                          (charsInBalanced '(' ')' litChar <|> count 1 litChar)
+                          (parenthesizedChars <|> count 1 litChar)
   let sourceURL = (unwords . words . concat) <$> many urlChunk
   let betweenAngles = try $
          char '<' >> manyTill litChar (char '>')
@@ -1497,7 +1515,7 @@ source = do
   return (escapeURI $ trimr src, tit)
 
 linkTitle :: MarkdownParser String
-linkTitle = fromEntities <$> (quotedTitle '"' <|> quotedTitle '\'')
+linkTitle = quotedTitle '"' <|> quotedTitle '\''
 
 link :: MarkdownParser (F Inlines)
 link = try $ do
@@ -1536,13 +1554,13 @@ referenceLink constructor (lab, raw) = do
     case M.lookup key keys of
        Nothing        -> do
          headers <- asksF stateHeaders
-         let ref' = B.toList $ runF (if labIsRef then lab else ref)
-                               defaultParserState
-         if implicitHeaderRefs && ref' `elem` headers
-            then do
-              let src = '#' : uniqueIdent ref' []
-              constructor src "" <$> lab
-            else (\x -> B.str "[" <> x <> B.str "]" <> B.str raw') <$> fallback
+         let ref' = runF (if labIsRef then lab else ref) defaultParserState
+         let makeFallback x = B.str "[" <> x <> B.str "]" <> B.str raw'
+         if implicitHeaderRefs
+            then case M.lookup ref' headers of
+                   Just ident -> constructor ('#':ident) "" <$> lab
+                   Nothing    -> makeFallback <$> fallback
+            else makeFallback <$> fallback
        Just (src,tit) -> constructor src tit <$> lab
 
 bareURL :: MarkdownParser (F Inlines)
@@ -1555,14 +1573,22 @@ autoLink :: MarkdownParser (F Inlines)
 autoLink = try $ do
   char '<'
   (orig, src) <- uri <|> emailAddress
-  char '>'
-  return $ return $ B.link src "" (B.str orig)
+  -- in rare cases, something may remain after the uri parser
+  -- is finished, because the uri parser tries to avoid parsing
+  -- final punctuation.  for example:  in `<http://hi---there>`,
+  -- the URI parser will stop before the dashes.
+  extra <- fromEntities <$> manyTill nonspaceChar (char '>')
+  return $ return $ B.link (src ++ escapeURI extra) "" (B.str $ orig ++ extra)
 
 image :: MarkdownParser (F Inlines)
 image = try $ do
   char '!'
   (lab,raw) <- reference
-  regLink B.image lab <|> referenceLink B.image (lab,raw)
+  defaultExt <- getOption readerDefaultImageExtension
+  let constructor src = case takeExtension src of
+                              "" -> B.image (addExtension src defaultExt)
+                              _  -> B.image src
+  regLink constructor lab <|> referenceLink constructor (lab,raw)
 
 note :: MarkdownParser (F Inlines)
 note = try $ do
@@ -1726,9 +1752,14 @@ singleQuoted = try $ do
     fmap B.singleQuoted . trimInlinesF . mconcat <$>
       many1Till inline singleQuoteEnd
 
+-- doubleQuoted will handle regular double-quoted sections, as well
+-- as dialogues with an open double-quote without a close double-quote
+-- in the same paragraph.
 doubleQuoted :: MarkdownParser (F Inlines)
 doubleQuoted = try $ do
   doubleQuoteStart
-  withQuoteContext InDoubleQuote $
-    fmap B.doubleQuoted . trimInlinesF . mconcat <$>
-      many1Till inline doubleQuoteEnd
+  contents <- mconcat <$> many (try $ notFollowedBy doubleQuoteEnd >> inline)
+  (withQuoteContext InDoubleQuote $ doubleQuoteEnd >> return
+       (fmap B.doubleQuoted . trimInlinesF $ contents))
+   <|> (return $ return (B.str "\8220") <> contents)
+
