@@ -92,7 +92,7 @@ import Text.Pandoc.MIME (getMimeType)
 import System.FilePath ( (</>), takeExtension, dropExtension )
 import Data.Generics (Typeable, Data)
 import qualified Control.Monad.State as S
-import Control.Monad (msum)
+import Control.Monad (msum, unless)
 import Text.Pandoc.Pretty (charWidth)
 import System.Locale (defaultTimeLocale)
 import Data.Time
@@ -413,8 +413,8 @@ isPara _        = False
 
 -- | Data structure for defining hierarchical Pandoc documents
 data Element = Blk Block
-             | Sec Int [Int] String [Inline] [Element]
-             --    lvl  num ident  label    contents
+             | Sec Int [Int] Attr [Inline] [Element]
+             --    lvl  num attributes label    contents
              deriving (Eq, Read, Show, Typeable, Data)
 
 -- | Convert Pandoc inline list to plain text identifier.  HTML
@@ -435,17 +435,19 @@ hierarchicalize blocks = S.evalState (hierarchicalizeWithIds blocks) []
 
 hierarchicalizeWithIds :: [Block] -> S.State [Int] [Element]
 hierarchicalizeWithIds [] = return []
-hierarchicalizeWithIds ((Header level (ident,_,_) title'):xs) = do
+hierarchicalizeWithIds ((Header level attr@(_,classes,_) title'):xs) = do
   lastnum <- S.get
   let lastnum' = take level lastnum
-  let newnum = if length lastnum' >= level
-                  then init lastnum' ++ [last lastnum' + 1]
-                  else lastnum ++ replicate (level - length lastnum - 1) 0 ++ [1]
-  S.put newnum
+  let newnum = case length lastnum' of
+                    x | "unnumbered" `elem` classes -> []
+                      | x >= level -> init lastnum' ++ [last lastnum' + 1]
+                      | otherwise -> lastnum ++
+                           replicate (level - length lastnum - 1) 0 ++ [1]
+  unless (null newnum) $ S.put newnum
   let (sectionContents, rest) = break (headerLtEq level) xs
   sectionContents' <- hierarchicalizeWithIds sectionContents
   rest' <- hierarchicalizeWithIds rest
-  return $ Sec level newnum ident title' sectionContents' : rest'
+  return $ Sec level newnum attr title' sectionContents' : rest'
 hierarchicalizeWithIds (x:rest) = do
   rest' <- hierarchicalizeWithIds rest
   return $ (Blk x) : rest'
