@@ -29,7 +29,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 Conversion of 'Pandoc' documents to docx.
 -}
 module Text.Pandoc.Writers.Docx ( writeDocx ) where
-import Data.List ( intercalate )
+import Data.List ( intercalate, groupBy )
 import qualified Data.ByteString as B
 import qualified Data.ByteString.Lazy as BL
 import qualified Data.Map as M
@@ -52,6 +52,8 @@ import Data.Unique (hashUnique, newUnique)
 import System.Random (randomRIO)
 import Text.Printf (printf)
 import qualified Control.Exception as E
+import System.FilePath (takeExtension)
+import Text.Pandoc.MIME (getMimeType)
 
 data WriterState = WriterState{
          stTextProperties :: [Element]
@@ -59,7 +61,7 @@ data WriterState = WriterState{
        , stFootnotes      :: [Element]
        , stSectionIds     :: [String]
        , stExternalLinks  :: M.Map String String
-       , stImages         :: M.Map FilePath (String, String, Element, B.ByteString)
+       , stImages         :: M.Map FilePath (String, String, Maybe String, Element, B.ByteString)
        , stListLevel      :: Int
        , stListNumId      :: Int
        , stNumStyles      :: M.Map ListMarker Int
@@ -112,40 +114,106 @@ writeDocx opts doc@(Pandoc (Meta tit auths date) _) = do
                        defaultWriterState
   epochtime <- floor `fmap` getPOSIXTime
   let imgs = M.elems $ stImages st
-  let toImgRel (ident,path,_,_) =  mknode "Relationship" [("Type","http://schemas.openxmlformats.org/officeDocument/2006/relationships/image"),("Id",ident),("Target",path)] ()
-  let newrels = map toImgRel imgs
-  let relpath = "word/_rels/document.xml.rels"
-  let reldoc = case findEntryByPath relpath refArchive >>=
-                    parseXMLDoc . UTF8.toStringLazy . fromEntry of
-                      Just d  -> d
-                      Nothing -> error $ relpath ++ "missing in reference docx"
-  let reldoc' = reldoc{ elContent = elContent reldoc ++ map Elem newrels }
-  -- create entries for images
-  let toImageEntry (_,path,_,img) = toEntry ("word/" ++ path) epochtime $ toLazy img
-  let imageEntries = map toImageEntry imgs
-  -- NOW get list of external links and images from this, and do what's needed
+
+  -- we create [Content_Types].xml and word/_rels/document.xml.rels
+  -- from scratch rather than reading from reference.docx,
+  -- because Word sometimes changes these files when a reference.docx is modified,
+  -- e.g. deleting the reference to footnotes.xml or removing default entries
+  -- for image content types.
+
+  -- [Content_Types].xml
+  let mkOverrideNode (part', contentType') = mknode "Override"
+               [("PartName",part'),("ContentType",contentType')] ()
+  let mkImageOverride (_, imgpath, mbMimeType, _, _) =
+             mkOverrideNode ("/word/" ++ imgpath, maybe "application/octet-stream" id mbMimeType)
+  let overrides = map mkOverrideNode
+                  [("/word/webSettings.xml",
+                    "application/vnd.openxmlformats-officedocument.wordprocessingml.webSettings+xml")
+                  ,("/word/numbering.xml",
+                    "application/vnd.openxmlformats-officedocument.wordprocessingml.numbering+xml")
+                  ,("/word/settings.xml",
+                    "application/vnd.openxmlformats-officedocument.wordprocessingml.settings+xml")
+                  ,("/word/theme/theme1.xml",
+                    "application/vnd.openxmlformats-officedocument.theme+xml")
+                  ,("/word/fontTable.xml",
+                    "application/vnd.openxmlformats-officedocument.wordprocessingml.fontTable+xml")
+                  ,("/docProps/app.xml",
+                    "application/vnd.openxmlformats-officedocument.extended-properties+xml")
+                  ,("/docProps/core.xml",
+                    "application/vnd.openxmlformats-package.core-properties+xml")
+                  ,("/word/styles.xml",
+                    "application/vnd.openxmlformats-officedocument.wordprocessingml.styles+xml")
+                  ,("/word/document.xml",
+                    "application/vnd.openxmlformats-officedocument.wordprocessingml.document.main+xml")
+                  ,("/word/footnotes.xml",
+                    "application/vnd.openxmlformats-officedocument.wordprocessingml.footnotes+xml")
+                  ] ++ map mkImageOverride imgs
+  let defaultnodes = [mknode "Default"
+              [("Extension","xml"),("ContentType","application/xml")] (),
+             mknode "Default"
+              [("Extension","rels"),("ContentType","application/vnd.openxmlformats-package.relationships+xml")] ()]
+  let contentTypesDoc = mknode "Types" [("xmlns","http://schemas.openxmlformats.org/package/2006/content-types")] $ defaultnodes ++ overrides
+  let contentTypesEntry = toEntry "[Content_Types].xml" epochtime
+        $ UTF8.fromStringLazy $ showTopElement' contentTypesDoc
+
+  -- word/_rels/document.xml.rels
+  let toBaseRel (url', id', target') = mknode "Relationship"
+                                          [("Type",url')
+                                          ,("Id",id')
+                                          ,("Target",target')] ()
+  let baserels = map toBaseRel
+                    [("http://schemas.openxmlformats.org/officeDocument/2006/relationships/numbering",
+                      "rId1",
+                      "numbering.xml")
+                    ,("http://schemas.openxmlformats.org/officeDocument/2006/relationships/styles",
+                      "rId2",
+                      "styles.xml")
+                    ,("http://schemas.openxmlformats.org/officeDocument/2006/relationships/settings",
+                      "rId3",
+                      "settings.xml")
+                    ,("http://schemas.openxmlformats.org/officeDocument/2006/relationships/webSettings",
+                      "rId4",
+                      "webSettings.xml")
+                    ,("http://schemas.openxmlformats.org/officeDocument/2006/relationships/fontTable",
+                      "rId5",
+                      "fontTable.xml")
+                    ,("http://schemas.openxmlformats.org/officeDocument/2006/relationships/theme",
+                      "rId6",
+                      "theme/theme1.xml")
+                    ,("http://schemas.openxmlformats.org/officeDocument/2006/relationships/footnotes",
+                      "rId7",
+                      "footnotes.xml")]
+  let toImgRel (ident,path,_,_,_) =  mknode "Relationship" [("Type","http://schemas.openxmlformats.org/officeDocument/2006/relationships/image"),("Id",ident),("Target",path)] ()
+  let imgrels = map toImgRel imgs
   let toLinkRel (src,ident) =  mknode "Relationship" [("Type","http://schemas.openxmlformats.org/officeDocument/2006/relationships/hyperlink"),("Id",ident),("Target",src),("TargetMode","External") ] ()
-  let newrels' = map toLinkRel $ M.toList $ stExternalLinks st
-  let reldoc'' = reldoc' { elContent = elContent reldoc' ++ map Elem newrels' }
-  let relEntry = toEntry relpath epochtime $ UTF8.fromStringLazy $ showTopElement' reldoc''
+  let linkrels = map toLinkRel $ M.toList $ stExternalLinks st
+  let reldoc = mknode "Relationships" [("xmlns","http://schemas.openxmlformats.org/package/2006/relationships")] $ baserels ++ imgrels ++ linkrels
+  let relEntry = toEntry "word/_rels/document.xml.rels" epochtime
+        $ UTF8.fromStringLazy $ showTopElement' reldoc
+
+  -- create entries for images in word/media/...
+  let toImageEntry (_,path,_,_,img) = toEntry ("word/" ++ path) epochtime $ toLazy img
+  let imageEntries = map toImageEntry imgs
+
+  -- word/document.xml
   let contentEntry = toEntry "word/document.xml" epochtime $ UTF8.fromStringLazy $ showTopElement' contents
+
   -- footnotes
   let footnotesEntry = toEntry "word/footnotes.xml" epochtime $ UTF8.fromStringLazy $
         showTopElement' footnotes
+
   -- footnote rels
   let footnoteRelEntry = toEntry "word/_rels/footnotes.xml.rels" epochtime $ UTF8.fromStringLazy $
         showTopElement' $ mknode "Relationships" [("xmlns","http://schemas.openxmlformats.org/package/2006/relationships")]
-        $ newrels'
+        $ linkrels
+
   -- styles
   let newstyles = styleToOpenXml $ writerHighlightStyle opts
   let stylepath = "word/styles.xml"
-  let styledoc = case findEntryByPath stylepath refArchive >>=
-                      parseXMLDoc . UTF8.toStringLazy . fromEntry of
-                        Just d  -> d
-                        Nothing -> error $ "Unable to parse " ++ stylepath ++
-                                           " from reference.docx"
+  styledoc <- parseXml refArchive stylepath
   let styledoc' = styledoc{ elContent = elContent styledoc ++ map Elem newstyles }
   let styleEntry = toEntry stylepath epochtime $ UTF8.fromStringLazy $ showTopElement' styledoc'
+
   -- construct word/numbering.xml
   let numpath = "word/numbering.xml"
   numEntry <- (toEntry numpath epochtime . UTF8.fromStringLazy . showTopElement')
@@ -167,13 +235,16 @@ writeDocx opts doc@(Pandoc (Meta tit auths date) _) = do
   rels <- case findEntryByPath relsPath refArchive of
                    Just e  -> return $ UTF8.toStringLazy $ fromEntry e
                    Nothing -> err 57 "could not find .rels/_rels in reference docx"
+
   -- fix .rels/_rels, which can get screwed up when reference.docx is edited by Word
   let rels' = substitute "http://schemas.openxmlformats.org/package/2006/relationships/metadata/core-properties"
                   "http://schemas.openxmlformats.org/officedocument/2006/relationships/metadata/core-properties"
                   rels
   let relsEntry = toEntry relsPath epochtime $ UTF8.fromStringLazy rels'
+
+  -- Create archive
   let archive = foldr addEntryToArchive refArchive $
-                  relsEntry : contentEntry : relEntry : footnoteRelEntry : numEntry : styleEntry : footnotesEntry : docPropsEntry : imageEntries
+                  contentTypesEntry : relsEntry : contentEntry : relEntry : footnoteRelEntry : numEntry : styleEntry : footnotesEntry : docPropsEntry : imageEntries
   return $ fromArchive archive
 
 styleToOpenXml :: Style -> [Element]
@@ -354,10 +425,16 @@ blockToOpenXML opts (Para [Image alt (src,'f':'i':'g':':':tit)]) = do
   captionNode <- withParaProp (pStyle "ImageCaption")
                  $ blockToOpenXML opts (Para alt)
   return $ mknode "w:p" [] (paraProps ++ contents) : captionNode
-blockToOpenXML opts (Para lst) = do
-  paraProps <- getParaProps
-  contents <- inlinesToOpenXML opts lst
-  return [mknode "w:p" [] (paraProps ++ contents)]
+blockToOpenXML opts (Para lst)
+  | any isDisplayMath lst && not (all isDisplayMath lst) = do
+    -- chop into several paragraphs so each displaymath is its own
+    let lsts = groupBy (\x y -> (isDisplayMath x && isDisplayMath y) ||
+                         not (isDisplayMath x || isDisplayMath y)) lst
+    blocksToOpenXML opts (map Para lsts)
+  | otherwise = do
+    paraProps <- getParaProps
+    contents <- inlinesToOpenXML opts lst
+    return [mknode "w:p" [] (paraProps ++ contents)]
 blockToOpenXML _ (RawBlock format str)
   | format == "openxml" = return [ x | Elem x <- parseXML str ]
   | otherwise           = return []
@@ -558,16 +635,13 @@ inlineToOpenXML opts (Quoted quoteType lst) =
     where (open, close) = case quoteType of
                             SingleQuote -> ("\x2018", "\x2019")
                             DoubleQuote -> ("\x201C", "\x201D")
-inlineToOpenXML opts (Math InlineMath str) =
-  case texMathToOMML DisplayInline str of
+inlineToOpenXML opts (Math mathType str) = do
+  let displayType = if mathType == DisplayMath
+                       then DisplayBlock
+                       else DisplayInline
+  case texMathToOMML displayType str of
         Right r -> return [r]
         Left  _ -> inlinesToOpenXML opts (readTeXMath str)
-inlineToOpenXML opts (Math DisplayMath str) =
-  case texMathToOMML DisplayBlock str of
-        Right r -> return [br, r, br]
-        Left  _ -> do
-            fallback <- inlinesToOpenXML opts (readTeXMath str)
-            return $ [br] ++ fallback ++ [br]
 inlineToOpenXML opts (Cite _ lst) = inlinesToOpenXML opts lst
 inlineToOpenXML _ (Code attrs str) =
   withTextProp (rStyle "VerbatimChar")
@@ -584,7 +658,7 @@ inlineToOpenXML opts (Note bs) = do
   notes <- gets stFootnotes
   notenum <- getUniqueId
   let notemarker = mknode "w:r" []
-                   [ mknode "w:rPr" [] (rStyle "FootnoteReference")
+                   [ mknode "w:rPr" [] (rStyle "FootnoteRef")
                    , mknode "w:footnoteRef" [] () ]
   let notemarkerXml = RawInline "openxml" $ ppElement notemarker
   let insertNoteRef (Plain ils : xs) = Plain (notemarkerXml : ils) : xs
@@ -601,15 +675,15 @@ inlineToOpenXML opts (Note bs) = do
   let newnote = mknode "w:footnote" [("w:id", notenum)] $ contents
   modify $ \s -> s{ stFootnotes = newnote : notes }
   return [ mknode "w:r" []
-           [ mknode "w:rPr" [] (rStyle "FootnoteReference")
+           [ mknode "w:rPr" [] (rStyle "FootnoteRef")
            , mknode "w:footnoteReference" [("w:id", notenum)] () ] ]
 -- internal link:
 inlineToOpenXML opts (Link txt ('#':xs,_)) = do
-  contents <- withTextProp (rStyle "Hyperlink") $ inlinesToOpenXML opts txt
+  contents <- withTextProp (rStyle "Link") $ inlinesToOpenXML opts txt
   return [ mknode "w:hyperlink" [("w:anchor",xs)] contents ]
 -- external link:
 inlineToOpenXML opts (Link txt (src,_)) = do
-  contents <- withTextProp (rStyle "Hyperlink") $ inlinesToOpenXML opts txt
+  contents <- withTextProp (rStyle "Link") $ inlinesToOpenXML opts txt
   extlinks <- gets stExternalLinks
   id' <- case M.lookup src extlinks of
             Just i   -> return i
@@ -623,7 +697,7 @@ inlineToOpenXML opts (Image alt (src, tit)) = do
   -- first, check to see if we've already done this image
   imgs <- gets stImages
   case M.lookup src imgs of
-    Just (_,_,elt,_) -> return [elt]
+    Just (_,_,_,elt,_) -> return [elt]
     Nothing -> do
       let sourceDir = writerSourceDirectory opts
       res <- liftIO $ E.try $ fetchItem sourceDir src
@@ -671,16 +745,33 @@ inlineToOpenXML opts (Image alt (src, tit)) = do
                   , mknode "wp:effectExtent" [("b","0"),("l","0"),("r","0"),("t","0")] ()
                   , mknode "wp:docPr" [("descr",tit),("id","1"),("name","Picture")] ()
                   , graphic ]
-          modify $ \st -> st{ stImages = M.insert src (ident, imgPath ident img, imgElt, img) $ stImages st }
-          return [imgElt]
-
-imgPath :: String -> B.ByteString -> String
-imgPath ident img = "media/" ++ ident ++
-                            case imageType img of
-                                  Just Png  -> ".png"
-                                  Just Jpeg -> ".jpeg"
-                                  Just Gif  -> ".gif"
-                                  Nothing   -> ""
+          let imgext = case imageType img of
+                             Just Png  -> ".png"
+                             Just Jpeg -> ".jpeg"
+                             Just Gif  -> ".gif"
+                             Just Pdf  -> ".pdf"
+                             Nothing   -> takeExtension src
+          if null imgext
+             then -- without an extension there is no rule for content type
+               inlinesToOpenXML opts alt -- return alt to avoid corrupted docx
+             else do
+               let imgpath = "media/" ++ ident ++ imgext
+               let mbMimeType = getMimeType imgpath
+               -- insert mime type to use in constructing [Content_Types].xml
+               modify $ \st -> st{ stImages =
+                   M.insert src (ident, imgpath, mbMimeType, imgElt, img)
+                           $ stImages st }
+               return [imgElt]
 
 br :: Element
 br = mknode "w:r" [] [mknode "w:cr" [] () ]
+
+parseXml :: Archive -> String -> IO Element
+parseXml refArchive relpath =
+  case (findEntryByPath relpath refArchive >>= parseXMLDoc . UTF8.toStringLazy . fromEntry) of
+       Just d  -> return d
+       Nothing -> fail $ relpath ++ " missing in reference docx"
+
+isDisplayMath :: Inline -> Bool
+isDisplayMath (Math DisplayMath _) = True
+isDisplayMath _                    = False
