@@ -62,6 +62,12 @@ import Text.Blaze.Html.Renderer.Utf8 (renderHtml)
 import Text.Blaze.Renderer.Utf8 (renderHtml)
 #endif
 
+-- A Chapter includes a list of blocks and maybe a section
+-- number offset.  Note, some chapters are unnumbered. The section
+-- number is different from the index number, which will be used
+-- in filenames, chapter0003.xhtml.
+data Chapter = Chapter (Maybe [Int]) [Block]
+
 -- | Produce an EPUB file from a Pandoc document.
 writeEPUB :: WriterOptions  -- ^ Writer options
           -> Pandoc         -- ^ Document to convert
@@ -126,7 +132,8 @@ writeEPUB opts doc@(Pandoc meta _) = do
   let blocks' = addIdentifiers
                 $ case blocks of
                       (Header 1 _ _ : _) -> blocks
-                      _                  -> Header 1 ("",[],[]) (docTitle meta) : blocks
+                      _                  -> Header 1 ("",["unnumbered"],[])
+                                                 (docTitle meta) : blocks
 
   let chapterHeaderLevel = writerEpubChapterLevel opts
   -- internal reference IDs change when we chunk the file,
@@ -138,22 +145,40 @@ writeEPUB opts doc@(Pandoc meta _) = do
   let isChapterHeader (Header n _ _) = n <= chapterHeaderLevel
       isChapterHeader _ = False
 
-  let toChunks :: [Block] -> [[Block]]
-      toChunks [] = []
-      toChunks (b:bs) = (b:xs) : toChunks ys
-        where (xs,ys) = break isChapterHeader bs
+  let toChapters :: [Block] -> State [Int] [Chapter]
+      toChapters []     = return []
+      toChapters (Header n attr@(_,classes,_) ils : bs) = do
+        nums <- get
+        mbnum <- if "unnumbered" `elem` classes
+                    then return Nothing
+                    else case splitAt (n - 1) nums of
+                              (ks, (m:_)) -> do
+                                let nums' = ks ++ [m+1]
+                                put nums'
+                                return $ Just (ks ++ [m])
+                                -- note, this is the offset not the sec number
+                              (ks, []) -> do
+                                let nums' = ks ++ [1]
+                                put nums'
+                                return $ Just ks
+        let (xs,ys) = break isChapterHeader bs
+        (Chapter mbnum (Header n attr ils : xs) :) `fmap` toChapters ys
+      toChapters (b:bs) = do
+        let (xs,ys) = break isChapterHeader bs
+        (Chapter Nothing (b:xs) :) `fmap` toChapters ys
 
-  let chunks = toChunks blocks''
+  let chapters = evalState (toChapters blocks'') []
 
-  let chapToEntry :: Int -> [Block] -> Entry
-      chapToEntry num bs = mkEntry (showChapter num)
+  let chapToEntry :: Int -> Chapter -> Entry
+      chapToEntry num (Chapter mbnum bs) = mkEntry (showChapter num)
         $ renderHtml
-        $ writeHtml opts'
+        $ writeHtml opts'{ writerNumberOffset =
+            maybe [] id mbnum }
         $ case bs of
               (Header _ _ xs : _) -> Pandoc (Meta xs [] []) bs
               _                   -> Pandoc (Meta [] [] []) bs
 
-  let chapterEntries = zipWith chapToEntry [1..] chunks
+  let chapterEntries = zipWith chapToEntry [1..] chapters
 
   -- incredibly inefficient (TODO):
   let containsMathML ent = "<math" `isInfixOf` (B8.unpack $ fromEntry ent)
@@ -204,12 +229,10 @@ writeEPUB opts doc@(Pandoc meta _) = do
                               ,("media-type","application/x-dtbncx+xml")] $ ()
              , unode "item" ! [("id","style"), ("href","stylesheet.css")
                               ,("media-type","text/css")] $ ()
-             ] ++
-             [ unode "item" ! [("id","nav")
-                              ,("href","nav.xhtml")
-                              ,("properties","nav")
-                              ,("media-type","application/xhtml+xml")] $ ()
-               | version == EPUB3
+             , unode "item" ! ([("id","nav")
+                               ,("href","nav.xhtml")
+                               ,("media-type","application/xhtml+xml")] ++
+                               [("properties","nav") | epub3 ]) $ ()
              ] ++
              map chapterNode (cpgEntry ++ (tpEntry : chapterEntries)) ++
              map pictureNode (cpicEntry ++ picEntries) ++
@@ -219,7 +242,15 @@ writeEPUB opts doc@(Pandoc meta _) = do
                     Nothing -> []
                     Just _ -> [ unode "itemref" !
                                 [("idref", "cover"),("linear","no")] $ () ]
-              ++ map chapterRefNode (tpEntry : chapterEntries)
+              ++ ((unode "itemref" ! [("idref", "title_page"),("linear","yes")] $ ()) :
+                  (unode "itemref" ! [("idref", "nav")
+                                     ,("linear", if writerTableOfContents opts
+                                                    then "yes"
+                                                    else "no")] $ ()) :
+                  map chapterRefNode chapterEntries)
+          , unode "guide" $
+              unode "reference" !
+                [("type","toc"),("title",plainTitle),("href","nav.xhtml")] $ ()
           ]
   let contentsEntry = mkEntry "content.opf" contentsData
 
@@ -230,13 +261,13 @@ writeEPUB opts doc@(Pandoc meta _) = do
 
   let navPointNode :: (Int -> String -> String -> [Element] -> Element)
                    -> Shared.Element -> State Int Element
-      navPointNode formatter (Sec _ nums ident ils children) = do
+      navPointNode formatter (Sec _ nums (ident,_,_) ils children) = do
         n <- get
         modify (+1)
         let showNums :: [Int] -> String
             showNums = intercalate "." . map show
         let tit' = plainify ils
-        let tit = if writerNumberSections opts
+        let tit = if writerNumberSections opts && not (null nums)
                      then showNums nums ++ " " ++ tit'
                      else tit'
         let src = case lookup ident reftable of
@@ -290,16 +321,19 @@ writeEPUB opts doc@(Pandoc meta _) = do
                                              $ (unode "span" tit))
                                             : case subs of
                                                  []    -> []
-                                                 (_:_) -> [unode "ol" subs]
+                                                 (_:_) -> [unode "ol" ! [("class","toc")] $ subs]
 
+  let navtag = if epub3 then "nav" else "div"
   let navData = fromStringLazy $ ppTopElement $
         unode "html" ! [("xmlns","http://www.w3.org/1999/xhtml")
                        ,("xmlns:epub","http://www.idpf.org/2007/ops")] $
-          [ unode "head" $ unode "title" plainTitle
+          [ unode "head" $
+            [ unode "title" plainTitle
+            , unode "link" ! [("rel","stylesheet"),("type","text/css"),("href","stylesheet.css")] $ () ]
           , unode "body" $
-              unode "nav" ! [("epub:type","toc")] $
+              unode navtag ! [("epub:type","toc") | epub3] $
                 [ unode "h1" plainTitle
-                , unode "ol" $ evalState (mapM (navPointNode navXhtmlFormatter) secs) 1]
+                , unode "ol" ! [("class","toc")] $ evalState (mapM (navPointNode navXhtmlFormatter) secs) 1]
           ]
   let navEntry = mkEntry "nav.xhtml" navData
 
@@ -332,9 +366,8 @@ writeEPUB opts doc@(Pandoc meta _) = do
   -- construct archive
   let archive = foldr addEntryToArchive emptyArchive
                  (mimetypeEntry : containerEntry : appleEntry : stylesheetEntry : tpEntry :
-                  contentsEntry : tocEntry :
-                  ([navEntry | version == EPUB3] ++ picEntries ++ cpicEntry ++ cpgEntry ++
-                    chapterEntries ++ fontEntries) )
+                  contentsEntry : tocEntry : navEntry :
+                  (picEntries ++ cpicEntry ++ cpgEntry ++ chapterEntries ++ fontEntries))
   return $ fromArchive archive
 
 metadataElement :: EPUBVersion -> String -> UUID -> String -> String -> [String]
@@ -360,7 +393,7 @@ metadataElement version metadataXML uuid lang title authors date currentTime mbC
                      $ a | a <- authors ] ++
            [ unode "dc:date" date | not (elt `contains` "date") ] ++
            [ unode "meta" ! [("property", "dcterms:modified")] $
-               (showDateTimeISO8601 currentTime) | version == EPUB3 ] ++
+               (showDateTimeISO8601 currentTime) | version == EPUB3] ++
            [ unode "meta" ! [("name","cover"), ("content","cover-image")] $ () |
                not (isNothing mbCoverImage) ]
   in  elt{ elContent = elContent elt ++ map Elem newNodes }
