@@ -250,11 +250,13 @@ showSecNum = concat . intersperse "." . map show
 -- | Converts an Element to a list item for a table of contents,
 -- retrieving the appropriate identifier from state.
 elementToListItem :: WriterOptions -> Element -> State WriterState (Maybe Html)
-elementToListItem opts (Sec lev num id' headerText subsecs)
+elementToListItem opts (Sec lev num (id',classes,_) headerText subsecs)
   | lev <= writerTOCDepth opts = do
-  let sectnum = if writerNumberSections opts
-                   then (H.span ! A.class_ "toc-section-number" $ toHtml $ showSecNum num) >>
-                     preEscapedString " "
+  let num' = zipWith (+) num (writerNumberOffset opts ++ repeat 0)
+  let sectnum = if writerNumberSections opts && not (null num) &&
+                   "unnumbered" `notElem` classes
+                   then (H.span ! A.class_ "toc-section-number"
+                        $ toHtml $ showSecNum num') >> preEscapedString " "
                    else mempty
   txt <- liftM (sectnum >>) $ inlineListToHtml opts headerText
   subHeads <- mapM (elementToListItem opts) subsecs >>= return . catMaybes
@@ -271,15 +273,16 @@ elementToListItem _ _ = return Nothing
 -- | Convert an Element to Html.
 elementToHtml :: Int -> WriterOptions -> Element -> State WriterState Html
 elementToHtml _slideLevel opts (Blk block) = blockToHtml opts block
-elementToHtml slideLevel opts (Sec level num id' title' elements) = do
+elementToHtml slideLevel opts (Sec level num (id',classes,keyvals) title' elements) = do
   let slide = writerSlideVariant opts /= NoSlides && level <= slideLevel
-  modify $ \st -> st{stSecNum = num}  -- update section number
+  let num' = zipWith (+) num (writerNumberOffset opts ++ repeat 0)
+  modify $ \st -> st{stSecNum = num'}  -- update section number
   -- always use level 1 for slide titles
   let level' = if slide then 1 else level
   let titleSlide = slide && level < slideLevel
   header' <- if title' == [Str "\0"]  -- marker for hrule
                 then return mempty
-                else blockToHtml opts (Header level' (id',[],[]) title')
+                else blockToHtml opts (Header level' (id',classes,keyvals) title')
   let isSec (Sec _ _ _ _ _) = True
       isSec (Blk _)         = False
   innerContents <- mapM (elementToHtml slideLevel opts)
@@ -287,22 +290,24 @@ elementToHtml slideLevel opts (Sec level num id' title' elements) = do
                         -- title slides have no content of their own
                         then filter isSec elements
                         else elements
-  let header'' = if (writerSectionDivs opts ||
-                     writerSlideVariant opts == S5Slides ||
-                     slide)
-                    then header'
-                    else header' ! prefixedId opts id'
   let inNl x = mconcat $ nl opts : intersperse (nl opts) x ++ [nl opts]
-  let classes = ["titleslide" | titleSlide] ++ ["slide" | slide] ++
-                  ["level" ++ show level]
+  let classes' = ["titleslide" | titleSlide] ++ ["slide" | slide] ++
+                  ["section" | (slide || writerSectionDivs opts) &&
+                               not (writerHtml5 opts) ] ++
+                  ["level" ++ show level | slide || writerSectionDivs opts ]
+                  ++ classes
   let secttag  = if writerHtml5 opts
-                    then H5.section ! A.class_ (toValue $ unwords classes)
-                    else H.div ! A.class_ (toValue $ unwords ("section":classes))
+                    then H5.section
+                    else H.div
+  let attr = (id',classes',keyvals)
   return $ if titleSlide
-              then mconcat $ (secttag ! prefixedId opts id' $ header'') : innerContents
+              then mconcat $
+                   (addAttrs opts attr $ secttag $ header') : innerContents
               else if writerSectionDivs opts || slide
-                   then secttag ! prefixedId opts id' $ inNl $ header'' : innerContents
-                   else mconcat $ intersperse (nl opts) $ header'' : innerContents
+                   then addAttrs opts attr
+                        $ secttag $ inNl $ header' : innerContents
+                   else mconcat $ intersperse (nl opts)
+                        $ addAttrs opts attr header' : innerContents
 
 -- | Convert list of Note blocks to a footnote <div>.
 -- Assumes notes are sorted.
@@ -371,10 +376,13 @@ obfuscateChar char =
 obfuscateString :: String -> String
 obfuscateString = concatMap obfuscateChar . fromEntities
 
+addAttrs :: WriterOptions -> Attr -> Html -> Html
+addAttrs opts attr h = foldl (!) h (attrsToHtml opts attr)
+
 attrsToHtml :: WriterOptions -> Attr -> [Attribute]
 attrsToHtml opts (id',classes',keyvals) =
-  [A.class_ (toValue $ unwords classes') | not (null classes')] ++
   [prefixedId opts id' | not (null id')] ++
+  [A.class_ (toValue $ unwords classes') | not (null classes')] ++
   map (\(x,y) -> customAttribute (fromString x) (toValue y)) keyvals
 
 imageExts :: [String]
@@ -420,16 +428,15 @@ blockToHtml opts (CodeBlock (id',classes,keyvals) rawCode) = do
                     then map (\c -> if map toLower c == "haskell"
                                        then "literatehaskell"
                                        else c) classes
-                    else filter (/= "literate") classes
+                    else classes
       adjCode  = if tolhs
                     then unlines . map ("> " ++) . lines $ rawCode
                     else rawCode
-  case highlight formatHtmlBlock (id',classes,keyvals) adjCode of
-         Nothing -> let attrs = attrsToHtml opts (id', classes', keyvals)
-                    in  return $ foldl (!) H.pre attrs $ H.code
-                                         $ toHtml adjCode
+  case highlight formatHtmlBlock (id',classes',keyvals) adjCode of
+         Nothing -> return $ addAttrs opts (id',classes,keyvals)
+                           $ H.pre $ H.code $ toHtml adjCode
          Just  h -> modify (\st -> st{ stHighlighting = True }) >>
-                    return (foldl (!) h (attrsToHtml opts (id',[],keyvals)))
+                    return (addAttrs opts (id',[],keyvals) h)
 blockToHtml opts (BlockQuote blocks) =
   -- in S5, treat list in blockquote specially
   -- if default is incremental, make it nonincremental;
@@ -451,22 +458,22 @@ blockToHtml opts (BlockQuote blocks) =
 blockToHtml opts (Header level (ident,_,_) lst) = do
   contents <- inlineListToHtml opts lst
   secnum <- liftM stSecNum get
-  let contents' = if writerNumberSections opts
-                     then (H.span ! A.class_ "header-section-number" $ toHtml $ showSecNum secnum) >>
-                            strToHtml " " >> contents
+  let contents' = if writerNumberSections opts && not (null secnum)
+                     then (H.span ! A.class_ "header-section-number" $ toHtml
+                          $ showSecNum secnum) >> strToHtml " " >> contents
                      else contents
   let contents''  = if writerTableOfContents opts && not (null ident)
                        then H.a ! A.href (toValue $
                               '#' : writerIdentifierPrefix opts ++ ident) $ contents'
                        else contents'
-  return $ (case level of
+  return $ case level of
               1 -> H.h1 contents''
               2 -> H.h2 contents''
               3 -> H.h3 contents''
               4 -> H.h4 contents''
               5 -> H.h5 contents''
               6 -> H.h6 contents''
-              _ -> H.p contents'')
+              _ -> H.p contents''
 blockToHtml opts (BulletList lst) = do
   contents <- mapM (blockListToHtml opts) lst
   let lst' = unordList opts contents
@@ -599,12 +606,11 @@ inlineToHtml opts inline =
     (Strong lst)     -> inlineListToHtml opts lst >>= return . H.strong
     (Code attr str)  -> case highlight formatHtmlInline attr str of
                              Nothing -> return
-                                        $ foldl (!) H.code (attrsToHtml opts attr)
-                                        $ strToHtml str
+                                        $ addAttrs opts attr
+                                        $ H.code $ strToHtml str
                              Just  h -> do
                                modify $ \st -> st{ stHighlighting = True }
-                               return $ foldl (!) h $
-                                          attrsToHtml opts (id',[],keyvals)
+                               return $ addAttrs opts (id',[],keyvals) h
                                 where (id',_,keyvals) = attr
     (Strikeout lst)  -> inlineListToHtml opts lst >>=
                         return . H.del
