@@ -35,18 +35,18 @@ import Text.Pandoc.Shared
 import Text.Pandoc.Options
 import Text.Pandoc.Templates
 import Text.Printf ( printf )
-import qualified Data.Map as M
 import Network.URI ( isAbsoluteURI, unEscapeString )
 import Data.List ( (\\), isSuffixOf, isInfixOf,
                    isPrefixOf, intercalate, intersperse )
 import Data.Char ( toLower, isPunctuation )
-import Control.Monad.State
 import Control.Applicative ((<|>))
+import Control.Monad.State
 import Text.Pandoc.Pretty
 import System.FilePath (dropExtension)
 import Text.Pandoc.Slides
 import Text.Pandoc.Highlighting (highlight, styleToLaTeX,
-                                 formatLaTeXInline, formatLaTeXBlock)
+                                 formatLaTeXInline, formatLaTeXBlock,
+                                 toListingsLanguage)
 
 data WriterState =
   WriterState { stInNote        :: Bool          -- true if we're in a note
@@ -171,8 +171,8 @@ pandocToLaTeX options (Pandoc (Meta title authors date) blocks) = do
 -- | Convert Elements to LaTeX
 elementToLaTeX :: WriterOptions -> Element -> State WriterState Doc
 elementToLaTeX _ (Blk block) = blockToLaTeX block
-elementToLaTeX opts (Sec level _ id' title' elements) = do
-  header' <- sectionHeader id' level title'
+elementToLaTeX opts (Sec level _ (id',classes,_) title' elements) = do
+  header' <- sectionHeader ("unnumbered" `elem` classes) id' level title'
   innerContents <- mapM (elementToLaTeX opts) elements
   return $ vsep (header' : innerContents)
 
@@ -236,7 +236,7 @@ toSlides bs = do
 
 elementToBeamer :: Int -> Element -> State WriterState [Block]
 elementToBeamer _slideLevel (Blk b) = return [b]
-elementToBeamer slideLevel  (Sec lvl _num ident tit elts)
+elementToBeamer slideLevel  (Sec lvl _num (ident,classes,_) tit elts)
   | lvl >  slideLevel = do
       bs <- concat `fmap` mapM (elementToBeamer slideLevel) elts
       return $ Para ( RawInline "latex" "\\begin{block}{"
@@ -244,7 +244,7 @@ elementToBeamer slideLevel  (Sec lvl _num ident tit elts)
              : bs ++ [RawBlock "latex" "\\end{block}"]
   | lvl <  slideLevel = do
       bs <- concat `fmap` mapM (elementToBeamer slideLevel) elts
-      return $ (Header lvl (ident,[],[]) tit) : bs
+      return $ (Header lvl (ident,classes,[]) tit) : bs
   | otherwise = do -- lvl == slideLevel
       -- note: [fragile] is required or verbatim breaks
       let hasCodeBlock (CodeBlock _ _) = [True]
@@ -332,11 +332,17 @@ blockToLaTeX (CodeBlock (_,classes,keyvalAttr) str) = do
                         then (case getListingsLanguage classes of
                                    Just l  -> [ "language=" ++ l ]
                                    Nothing -> []) ++
-                             [ key ++ "=" ++ attr | (key,attr) <- keyvalAttr ]
+                             [ "numbers=left" | "numberLines" `elem` classes
+                                || "number" `elem` classes
+                                || "number-lines" `elem` classes ] ++
+                             [ (if key == "startFrom"
+                                   then "firstnumber"
+                                   else key) ++ "=" ++ attr |
+                                   (key,attr) <- keyvalAttr ]
                         else []
                printParams
                    | null params = empty
-                   | otherwise   = brackets $ hsep (intersperse "," (map text params))
+                   | otherwise   = brackets $ hcat (intersperse ", " (map text params))
            return $ flush ("\\begin{lstlisting}" <> printParams $$ text str $$
                     "\\end{lstlisting}") $$ cr
          highlightedCodeBlock =
@@ -346,6 +352,7 @@ blockToLaTeX (CodeBlock (_,classes,keyvalAttr) str) = do
                              return (flush $ text h)
 blockToLaTeX (RawBlock "latex" x) = return $ text x
 blockToLaTeX (RawBlock _ _) = return empty
+blockToLaTeX (BulletList []) = return empty  -- otherwise latex error
 blockToLaTeX (BulletList lst) = do
   incremental <- gets stIncremental
   let inc = if incremental then "[<+->]" else ""
@@ -355,6 +362,7 @@ blockToLaTeX (BulletList lst) = do
                    else empty
   return $ text ("\\begin{itemize}" ++ inc) $$ spacing $$ vcat items $$
              "\\end{itemize}"
+blockToLaTeX (OrderedList _ []) = return empty -- otherwise latex error
 blockToLaTeX (OrderedList (start, numstyle, numdelim) lst) = do
   st <- get
   let inc = if stIncremental st then "[<+->]" else ""
@@ -393,6 +401,7 @@ blockToLaTeX (OrderedList (start, numstyle, numdelim) lst) = do
          $$ spacing
          $$ vcat items
          $$ "\\end{enumerate}"
+blockToLaTeX (DefinitionList []) = return empty
 blockToLaTeX (DefinitionList lst) = do
   incremental <- gets stIncremental
   let inc = if incremental then "[<+->]" else ""
@@ -404,7 +413,8 @@ blockToLaTeX (DefinitionList lst) = do
                "\\end{description}"
 blockToLaTeX HorizontalRule = return $
   "\\begin{center}\\rule{3in}{0.4pt}\\end{center}"
-blockToLaTeX (Header level (id',_,_) lst) = sectionHeader id' level lst
+blockToLaTeX (Header level (id',classes,_) lst) =
+  sectionHeader ("unnumbered" `elem` classes) id' level lst
 blockToLaTeX (Table caption aligns widths heads rows) = do
   modify $ \s -> s{ stInTable = True, stTableNotes = [] }
   headers <- if all null heads
@@ -475,15 +485,17 @@ defListItemToLaTeX (term, defs) = do
     return $ "\\item" <> brackets term' $$ def'
 
 -- | Craft the section header, inserting the secton reference, if supplied.
-sectionHeader :: [Char]
+sectionHeader :: Bool    -- True for unnumbered
+              -> [Char]
               -> Int
               -> [Inline]
               -> State WriterState Doc
-sectionHeader ref level lst = do
+sectionHeader unnumbered ref level lst = do
   txt <- inlineListToLaTeX lst
   let noNote (Note _) = Str ""
       noNote x        = x
   let lstNoNotes = bottomUp noNote lst
+  let star = if unnumbered then text "*" else empty
   -- footnotes in sections don't work unless you specify an optional
   -- argument:  \section[mysec]{mysec\footnote{blah}}
   optional <- if lstNoNotes == lst
@@ -491,7 +503,7 @@ sectionHeader ref level lst = do
                  else do
                    res <- inlineListToLaTeX lstNoNotes
                    return $ char '[' <> res <> char ']'
-  let stuffing = optional <> char '{' <> txt <> char '}'
+  let stuffing = star <> optional <> braces txt
   book <- gets stBook
   opts <- gets stOptions
   let level' = if book || writerChapters opts then level - 1 else level
@@ -504,17 +516,23 @@ sectionHeader ref level lst = do
                                                <> braces (text ref))
                          else lab)
   let headerWith x y = refLabel $ text x <> y
-  return $ case level' of
-                0  -> if writerBeamer opts
-                         then headerWith "\\part" stuffing
-                         else headerWith "\\chapter" stuffing
-                1  -> headerWith "\\section" stuffing
-                2  -> headerWith "\\subsection" stuffing
-                3  -> headerWith "\\subsubsection" stuffing
-                4  -> headerWith "\\paragraph" stuffing
-                5  -> headerWith "\\subparagraph" stuffing
-                _            -> txt
-
+  let sectionType = case level' of
+                          0  | writerBeamer opts -> "part"
+                             | otherwise -> "chapter"
+                          1  -> "section"
+                          2  -> "subsection"
+                          3  -> "subsubsection"
+                          4  -> "paragraph"
+                          5  -> "subparagraph"
+                          _  -> ""
+  return $ if level' > 5
+              then txt
+              else headerWith ('\\':sectionType) stuffing
+                   $$ if unnumbered
+                         then "\\addcontentsline{toc}" <>
+                                braces (text sectionType) <>
+                                braces txt
+                         else empty
 
 -- | Convert list of inline elements to LaTeX.
 inlineListToLaTeX :: [Inline]  -- ^ Inlines to convert
@@ -744,60 +762,7 @@ citationsToBiblatex (c:cs) = do
 
 citationsToBiblatex _ = return empty
 
--- correlate pandoc language names with listings names
-langsMap :: M.Map String String
-langsMap = M.fromList
-               [("ada","Ada")
-               ,("java","Java")
-               ,("prolog","Prolog")
-               ,("python","Python")
-               ,("gnuassembler","Assembler")
-               ,("commonlisp","Lisp")
-               ,("r","R")
-               ,("awk","Awk")
-               ,("bash","bash")
-               ,("makefile","make")
-               ,("c","C")
-               ,("matlab","Matlab")
-               ,("ruby","Ruby")
-               ,("cpp","C++")
-               ,("ocaml","Caml")
-               ,("modula2","Modula-2")
-               ,("sql","SQL")
-               ,("eiffel","Eiffel")
-               ,("tcl","tcl")
-               ,("erlang","erlang")
-               ,("verilog","Verilog")
-               ,("fortran","Fortran")
-               ,("vhdl","VHDL")
-               ,("pascal","Pascal")
-               ,("perl","Perl")
-               ,("xml","XML")
-               ,("haskell","Haskell")
-               ,("php","PHP")
-               ,("xslt","XSLT")
-               ,("html","HTML")
-               ]
-
-listingsLangs :: [String]
-listingsLangs = ["Ada","Java","Prolog","Algol","JVMIS","Promela",
-                 "Ant","ksh","Python","Assembler","Lisp","R","Awk",
-                 "Logo","Reduce","bash","make","Rexx","Basic",
-                 "Mathematica","RSL","C","Matlab","Ruby","C++",
-                 "Mercury","S","Caml","MetaPost","SAS","Clean",
-                 "Miranda","Scilab","Cobol","Mizar","sh","Comal",
-                 "ML","SHELXL","csh","Modula-2","Simula","Delphi",
-                 "MuPAD","SQL","Eiffel","NASTRAN","tcl","Elan",
-                 "Oberon-2","TeX","erlang","OCL","VBScript","Euphoria",
-                 "Octave","Verilog","Fortran","Oz","VHDL","GCL",
-                 "Pascal","VRML","Gnuplot","Perl","XML","Haskell",
-                 "PHP","XSLT","HTML","PL/I"]
-
 -- Determine listings language from list of class attributes.
 getListingsLanguage :: [String] -> Maybe String
 getListingsLanguage [] = Nothing
-getListingsLanguage (x:xs) = (if x `elem` listingsLangs
-                              then Just x
-                              else Nothing) <|>
-                             M.lookup (map toLower x) langsMap <|>
-                             getListingsLanguage xs
+getListingsLanguage (x:xs) = toListingsLanguage x <|> getListingsLanguage xs
